/*
 * ICQNET.C - Funcoes de rede para o pwICQ
 *
 */

/* Uncomment the line below to compile using SOCKS5 toolkit (It failed in my tests-PW) */
/* #define SOCKS5 */

/* Uncomment the line below to use the Old TCP-IP stack */
/* #define TCPV40HDRS */

 #define INCL_WIN
 #define INCL_DOSQUEUES

 #ifdef __BORLANDC__
    #include <bcos2.h>
 #else
    #include <os2.h>
 #endif

 #ifndef BUILD
    #define BUILD __DATE__ " " __TIME__
 #endif

 #define __ssize_t /* To satisfy the TCP-IP 4.21 toolkit */

 #ifdef __BORLANDC__
    #define __STDC__
 #endif

 #include <stdio.h>

 #define BSD_SELECT
 #include <sys/time.h>
 #include <types.h>
 #include <unistd.h>
 #include <sys/socket.h>
 #include <sys/sysctl.h>
 #include <netdb.h>
 #include <netinet/in.h>
 #include <net/if.h>
 #include <sys/ioctl.h>
 #include <arpa/inet.h>

 #ifdef __BORLANDC__
    #undef __STDC__
 #endif

 #include <stdlib.h>
 #include <string.h>

 #include <icqkernel.h>
 #include <pwicqnet.h>
 #include <icqplug.h>

 #ifdef SOCKS5
    #include <libsock5.h>
 #endif

 #include <resource.h>

 #include <pwmacros.h>

 #ifdef __BORLANDC__
    #pragma warn -parm
    #pragma warn -stu
 #endif

/*---[ Implementation ]---------------------------------------------------------*/

#ifdef __DLL__
   extern HMODULE module = NULLHANDLE;
#endif

#ifndef __BORLANDC__
   #ifdef SOCKS5
      BLDLEVEL(_PWICQVERSION_,"Socks5 module to pwICQ");
   #else
      BLDLEVEL(_PWICQVERSION_,"Network services to pwICQ");
   #endif
#endif

#ifdef __BORLANDC__

   #ifdef __DLL__

   /* Borland C startup code */

   ULONG _dllmain(ULONG term, HMODULE mod)
   {
      if(term)
         module = NULLHANDLE;
      else
         module = mod;
      return 1;
   }

   #endif

#else

/* Visual-Age C startup code */
unsigned long _System _DLL_InitTerm(unsigned long hModule, unsigned long ulFlag)
{
   switch (ulFlag)
   {
   case 0 :
      if (_CRT_init() == -1)
         return 0UL;
      break;

   case 1 :
      break;

   default  :
      return 0UL;
   }

   module = hModule;
   return 1UL;
}

#endif

int EXPENTRY ICQCloseSocket(int sock)
{
   int ret = 0;

   if(sock > 0)
   {
      ret = shutdown(sock,2);
      if(ret >= 0)
         ret = soclose(sock);
   }
   return ret;
}

int EXPENTRY ICQCancelSocket(int sock)
{
   int ret = 0;

   if(sock > 0)
   {
      ret = so_cancel(sock);

      #ifdef _DBG_MODE_
         if(ret < 0)
         {
            DBGTrace(sock_errno());
            psock_errno(NULL);
         }
      #endif

   }
   return ret;
}

long EXPENTRY ICQGetHostID()
{
   long ret = htonl(gethostid());

   if(ret == 0 || ret == 0xFFFFFFFF)
      ret = loopback;

   return ret;
}

long EXPENTRY ICQGetHostByName(char *hostname)
{
   struct hostent *host;

   host = gethostbyname(hostname);

   if(host == NULL)
      return 0;

   return *( (long *) host->h_addr );

}

int EXPENTRY ICQGetPort(int sock)
{
   struct sockaddr_in   in;
   int                  size;

   if(sock < 1)
      return 0;

   size = sizeof(in);
   if(getsockname(sock,(struct sockaddr *) &in, &size) == -1)
      return -1;

   return htons(in.sin_port);
}

int EXPENTRY ICQSendTo(int sock, long ip, int port, void *pkt, int size)
{
   struct sockaddr_in   to;

   to.sin_family      = AF_INET;
   to.sin_addr.s_addr = ip;
   to.sin_port        = port;

   if(sendto(sock,(char *) pkt,size,0,(struct sockaddr *) &to, sizeof(to)) == -1)
      return sock_errno();

   return 0;
}

int EXPENTRY ICQGetSockError(void)
{
   return sock_errno();
}

int EXPENTRY ICQRecvFrom(int sock, long *ip, int *port, void *buffer, int size)
{
   struct sockaddr_in   addr;
   int                  wrk;

   wrk  = sizeof(addr);
   memset(&addr,0,wrk);

   size = recvfrom(     sock,
                        (char *) buffer,
                        size,
                        0,
                        (struct sockaddr *) &addr,
                        &wrk);


   *ip   = addr.sin_addr.s_addr;
   *port = htons(addr.sin_port);

   return size;

}

int EXPENTRY ICQOpenUDP(void)
{
   return socket(PF_INET,SOCK_DGRAM,0);
}

int EXPENTRY ICQOpenTCP(void)
{
   struct linger        l;
   int                  ret = socket(PF_INET,SOCK_STREAM,0);

   if(ret < 0)
      return ret;

   l.l_onoff  = 0;
   l.l_linger = 1;

   if(setsockopt(ret,SOL_SOCKET,SO_LINGER,(char *) &l, sizeof(struct linger)) == -1)
   {
      ICQCloseSocket(ret);
      return -1;
   }

   return ret;
}

int EXPENTRY ICQBindSocket(int sock, int port, int *bnd)
{
   int                  ret;
   struct sockaddr_in   in;

   DBGTrace(sock);
   DBGTrace(port);

   memset(&in, 0, sizeof(in));

   in.sin_family                = AF_INET;
   in.sin_addr.s_addr           = INADDR_ANY;
   in.sin_port                  = htons(port);

   ret = bind(sock, (struct sockaddr *) &in, sizeof(in));

   if(bnd)
      *bnd = ICQGetPort(sock);

   return ret;
}

int EXPENTRY ICQAcceptConnct(int sock, long *ip, int *port)
{
   int                  size;
   int                  ret;
   struct sockaddr_in   addr;

   memset(&addr,0,sizeof(addr));

   size   = sizeof(addr);
   ret    = accept(     sock,
                        (struct sockaddr *) &addr,
                        &size );

   *ip   = addr.sin_addr.s_addr;
   *port = addr.sin_port;

   DBGTrace(sock);
   DBGTrace(ret);

   return ret;
}

int EXPENTRY ICQSetSockTimeout(int sock, HICQ icq, const char *tag, int def)
{
   struct timeval t;

   t.tv_sec = def;

   if(tag && icq)
      t.tv_sec = ICQLoadLong(icq,tag,def);

   t.tv_usec = 0;
   if(setsockopt(sock, SOL_SOCKET, 0x1006, (char *) &t, sizeof(t)) != 0)
   {
      if(icq)
         ICQWriteSysLog(icq,"Error setting socket timeout");
      return -1;
   }
   return 0;
}

int EXPENTRY ICQListen(int sock)
{
   return listen(sock,5);
}

int EXPENTRY ICQSetTCPOpt(int sock)
{
   int size = 1;
   return setsockopt(sock,SOL_SOCKET,SO_KEEPALIVE,(char *) &size, sizeof(size));
}

int EXPENTRY ICQRecv(int sock, void *ptr, int size)
{
   return recv(sock,(char *) ptr, size, 0);
}

int EXPENTRY ICQSend(int sock, void *ptr, int size)
{
   if(size == -1)
      size = strlen(ptr);
   return send(sock, (char *) ptr, size, 0);
}

int EXPENTRY ICQConnect(int sock, long ip, int port)
{
   struct sockaddr_in   addr;

   if(!ip)
      return -1;

   memset(&addr,0,sizeof(addr));

   addr.sin_family      = AF_INET;
   addr.sin_addr.s_addr = ip;
   addr.sin_port        = htons(port);

   return connect(sock, (struct sockaddr *) &addr, sizeof(addr));

}

#ifdef SOCKS5

BOOL EXPENTRY chkForNat(HICQ ctl, long ip)
{
   return TRUE;
}

#else

BOOL EXPENTRY chkForNat(HICQ ctl, long ip)
{
   #ifdef _DBG_MODE_
      char dbg[0x0100];
   #endif
   char                 buffer[0x0100];
   struct ifreq         i;
   struct sockaddr_in  *a;
   char                 *ptr;
   char                 *wrk;

   ICQLoadString(ctl,"Interfaces",0x0100,buffer,"");
   DBGMessage(buffer);

   if(!*buffer)
      return FALSE;

   ptr = buffer;
   while(*ptr)
   {
      for(wrk=ptr;*ptr && *ptr != ',';ptr++);
      if(*ptr)
         *(ptr++) = 0;
      DBGMessage(wrk);

      strncpy(i.ifr_name,wrk,IFNAMSIZ);

      if(ioctl(ctl->UDPsock,SIOCGIFADDR,(caddr_t) &i, sizeof(i)) != -1)
      {
         a = (struct sockaddr_in *) &i.ifr_addr;
         if(a->sin_addr.s_addr == ip)
            return FALSE;
      }
   }

   CHKPoint();

   return TRUE;
}
#endif

BOOL EXPENTRY ICQCheckVersion(HICQ ctl)
{
   /* Returns TRUE if the detected MPTS version is valid */

   #ifndef TCPV40HDRS

      char                      wrk[80];
      int                       mib[4] = { CTL_KERN,KERN_HOSTID,IPPROTO_IP,KERNCTL_INETVER };
      struct inetvers_ctl       vrs;
      size_t                    sz    = sizeof(vrs);
      int                       rc;

      rc = sysctl(mib, 4, &vrs, &sz, NULL, 0);

      if(rc == -1)
      {
         sprintf(wrk,"Error %d in sysctl()",sock_errno());
         DBGMessage(wrk);
         ICQWriteSysLog(ctl,wrk);
         return FALSE;
      }
      else
      {
         DBGMessage("Sysctl ok!");
         DBGMessage(vrs.versionstr);

         strcpy(wrk,"Detected MPTS ");
         strncat(wrk,vrs.versionstr,79);
         strncat(wrk," and ICQNET32 Build " BUILD,79);
         #ifdef _DBG_MODE_
            strncat(wrk," [DEBUG]",79);
         #endif
         ICQWriteSysLog(ctl,wrk);

         rc = atoi(vrs.versionstr);
         DBGTrace(rc);

         if(rc < 6)
         {
            DBGMessage("****************** MPTS Invalido!!!");
            ICQWriteSysLog(ctl,"Invalid MPTS version");
            return FALSE;
         }

      }

   #else

      ICQWriteSysLog(ctl,"Using ICQNET16 Build " BUILD);

   #endif

   return TRUE;
}

BOOL EXPENTRY ICQQueryTCP32(void)
{
#ifdef TCPV40HDRS
   return FALSE;
#else
   return TRUE;
#endif
}

int EXPENTRY ICQVerifySocket(HICQ ctl, int sock)
{
   /*
    * Check for socket activity, returns bitmask identifier
    *
    * 0x0001 = Read
    * 0x0002 = Write
    *
    */

#ifndef TCPV40HDRS

   int ret   = 0;
   int bytes = 0;

   if(ioctl(sock, FIONREAD , (caddr_t) &bytes))
      return 0x8000;

   if(bytes)
      ret |= 0x00001;

   return ret;

#else

   #error "Verify this part of code; I'm not sure if it works"

   fd_set               rd;
   fd_set               wr;
   struct timeval       tv;
   int                  ret = 0;

   memset(&tv,0,sizeof(struct timeval));

   FD_ZERO(&rd);
   FD_ZERO(&wr);

   FD_SET(sock,&rd);
   FD_SET(sock,&wr);

   switch(select(FD_SETSIZE,&rd,&wr,NULL,&tv))
   {
   case -1:
      return 0x8000;

   case 0:
      return 0x0000;

   }

   if(FD_ISSET(sock,&rd))
      ret |= 0x0001;

   if(FD_ISSET(sock,&wr))
      ret |= 0x0002;

   return ret;

#endif

}

const char * EXPENTRY ICQFormatIP(long ip)
{
   struct in_addr in;
   in.s_addr = ip;
   return inet_ntoa(in);
}


