/*
 * ICQPLAY.C - Toca um .wav
 */

 #define INCL_OS2MM
 #define INCL_MCIOS2
 #define INCL_DOSMODULEMGR
 #define INCL_DOSPROCESS
 #include <os2.h>

 #include <string.h>
 #include <os2me.h>
 #include <mciapi.h>
 #include <stdio.h>
 #include <stdlib.h>

 #include <pwmacros.h>

 #include <pwicqdef.h>
 #include <icqplug.h>

/*---[ Definicoes ]-------------------------------------------------------------*/

 struct parm
 {
    ULONG       uin;
    int         share;
    char        fileName[0x0100];
 };


/* #define NOPLAY */

 #define WAVALIAS "%08lx.%02x"

/*---[ Prototipos ]-------------------------------------------------------------*/

static void _System     mainThread(struct parm *);
static APIRET           execMCI(PSZ);

/*---[ Variaveis estaticas ]----------------------------------------------------*/

static TID      thread   = 0;
extern HMODULE  module   = NULLHANDLE;
static UCHAR    Sequence = 0;

/*---[ Implementacao ]----------------------------------------------------------*/

unsigned long _System _DLL_InitTerm(unsigned long hModule, unsigned long ulFlag)
{
   switch (ulFlag)
   {
   case 0 :
      if (_CRT_init() == -1)
         return 0UL;
      break;

   case 1 :
      break;

   default  :
      return 0UL;
   }

   module = hModule;
   return 1UL;
}

void EXPENTRY ICQPlayInitialize(HICQ ctl, HMODULE mod)
{
   ICQWriteSysLog(ctl,PROJECT " Started (" BUILD
#ifdef DEBUG
      " - DEBUG"
#endif
#ifdef NOPLAY
      " - ******** NOT USING MMOS2 - AUDIO DISABLED *******"
#endif
        ")" );

#ifdef DEBUG
   printf(PROJECT ": Running in debug mode");
#endif


}

void EXPENTRY ICQPlayTerminate(HICQ ctl)
{
   int f;

   for(f=0;f<100&&thread;f++)
      DosSleep(10);

   if(thread)
      ICQWriteSysLog(ctl,PROJECT ": Failure disabling Multimedia thread");
   else
      ICQWriteSysLog(ctl,PROJECT ": Multimedia thread deactivated");
}

void EXPENTRY ICQKeySound(HICQ ctl, char key)
{

}

int EXPENTRY ICQPlayWav(HICQ ctl, const char *sound)
{
   static struct parm   p;
   APIRET               rc;
   ULONG                uin = ICQQueryUin(ctl);
   char                 lin[70];

   if(!uin)
      return 0;

   DosEnterCritSec();

   if(thread || !*sound || p.uin)
   {
      DosExitCritSec();
      return 0;
   }
   p.uin   = uin;
   DosExitCritSec();

   Sequence++;

   p.share = ICQLoadLong(ctl,"SNDShare" ,1);
   strncpy(p.fileName,sound,0xFF);

   DBGMessage(sound);
   DBGTrace(p.uin);
   DBGTracex(&p);

   rc = DosCreateThread(&thread, (PFNTHREAD) mainThread, (ULONG) &p, 0x00000002, 32768);

   DBGTrace(rc);

   if(rc)
   {
      sprintf(lin,PROJECT ": SYS%04d starting thread",rc);
      ICQWriteSysLog(ctl,lin);
   }
   else
   {
      for(rc=0;rc<100 && p.uin;rc++)
         DosSleep(10);

      DBGTrace(p.uin);

      if(p.uin)
         ICQWriteSysLog(ctl,PROJECT ": Error passing information to the service thread");

   }

   return 0;
}

MRESULT EXPENTRY ICQMMMsg(HICQ ctl, HWND hwnd, ULONG msg, MPARAM mp1, MPARAM mp2)
{
   CHKPoint();
   return 0;
}

static void _System mainThread(struct parm *p)
{
   #pragma handler ( _Exception )

   ULONG        uin;
   HAB		hab;
   HMQ		hmq;
   int          share;
   UCHAR        counter         = Sequence;
   static char  fileName[0x0100];
   char         linha[0xFF];

   DBGTracex(p);

   if(!p)
      return;

   strncpy(fileName,p->fileName,0xFF);
   uin    = p->uin;
   share  = p->share;
   p->uin = 0;

   hab = WinInitialize(0);
   DBGTracex(hab);

   if(hab)
   {

      hmq = WinCreateMsgQueue( hab, 0 );
      DBGTracex(hmq);

      if(hmq)
      {	
         DBGMessage(fileName);

         sprintf(linha,"open %s%s type waveaudio alias " WAVALIAS " wait",
                              fileName,
                              share ? " shareable" : "",
                              uin, counter);
         execMCI(linha);

         sprintf(linha,"play " WAVALIAS " wait", uin, counter);
         execMCI(linha);

         sprintf(linha,"close " WAVALIAS " wait" , uin, counter);
         execMCI(linha);

         WinDestroyMsgQueue(hmq);
      }

      WinTerminate(hab);
   }

   thread = 0;
   CHKPoint();

   _endthread();

}

static APIRET execMCI(PSZ cmd)
{

   DBGMessage(cmd);

#ifdef NOPLAY
   return 0;
#else
   return mciSendString (       (PSZ) cmd,            /* Comando */
                                0,                    /* Buffer de retorno */
                                0,                    /* Tamanho do buffer de retorno */
                                0,                    /* Janela que recebe as notificacoes */
                                0 );                  /* Identificacao do servico */
#endif

}



