/*
 * QSEND.C - Functions to interface with pwICQ
 */

#define INCL_DOSQUEUES
#define INCL_DOSPROCESS

#include <string.h>
#include <pwicqapi.h>

#include "icqqueue.h"

/*---[ Prototipes ]-----------------------------------------------------------*/

static char *insertString(char *, int *, const char *);

/*---[ Implementation ]-------------------------------------------------------*/

APIRET icqSendULongCMD(const char *uin, char cmd, int qty, const ULONG *vlr)
{
   /*
    * Send a single command with "qty" ULONG parameters, wait for the response in the first pointer
    */
   ULONG     sz         = 3 + (qty*4);
   char      *mem;
   char      *ptr;
   ULONG     *p;
   int       f;
   APIRET    rc;

   rc = DosAllocSharedMem((PVOID) &mem, NULL, sz, OBJ_GETTABLE|OBJ_GIVEABLE|PAG_COMMIT|PAG_READ|PAG_WRITE);
   if(rc)
      return rc;

   /* 2 bytes for the return code */
   ptr = mem+2;

   /* The command request */
   *(ptr++) = cmd;

   /* The parameters  */
   p = (ULONG *) ptr;

   for(f=0;f<qty;f++)
      *(p+f) = *(vlr+f);

   rc = icqSendMemBlock(uin, sz, mem);

   DosFreeMem(mem);
   return rc;

}

APIRET icqSendUserInfo(const char *to, char cmd, ULONG uin, const char *nick, const char *email, const char *first, const char *last)
{
   int          sz      = 8;
   APIRET       rc;
   char         *mem;
   char         *ptr;

   insertString(NULL, &sz, nick);
   insertString(NULL, &sz, email);
   insertString(NULL, &sz, first);
   insertString(NULL, &sz, last);

   rc = DosAllocSharedMem((PVOID) &mem, NULL, sz, OBJ_GETTABLE|OBJ_GIVEABLE|PAG_COMMIT|PAG_READ|PAG_WRITE);
   if(rc)
      return rc;

   /* 2 bytes for the return code */
   ptr = mem+2;

   /* The command request */
   *(ptr++) = cmd;

   /* The user UIN */
   *( (ULONG *) ptr ) = uin;
   ptr += 4;

   ptr = insertString(ptr, NULL, nick);
   ptr = insertString(ptr, NULL, email);
   ptr = insertString(ptr, NULL, first);
   insertString(ptr, NULL, last);

   rc = icqSendMemBlock(to, sz, mem);

   DosFreeMem(mem);
   return rc;
}

static char *insertString(char *dst, int *qtd, const char *src)
{
   USHORT sz;

   if(!src)
      src = "";

   sz = strlen(src)+1;

   if(dst)
   {
      *( (USHORT *) dst ) = sz;
      dst += 2;
      strcpy(dst,src);
      dst += sz;
   }

   if(qtd)
      (*qtd) += sz+2;

   return dst;

}


