/*
 * ICQQueue.H - Funcoes to send/receive messages from the pwICQ's queue
 */

/*---[ Queue commands ]-------------------------------------------------------*/

 #define ICQCMD_RESTART         0x0001
 #define ICQCMD_SHOW            0x0002
 #define ICQCMD_QUIT            0x0003
 #define ICQCMD_SETMODE         0x0004
 #define ICQCMD_BLOCK           0x0005
 #define ICQCMD_OPEN            0x0006
 #define ICQCMD_ACTION          0x0007  /* Remote action request              */
 #define ICQCMD_OPENLIST        0x0008  /* Open userlist dialog box           */
 #define ICQCMD_VISIBLE		0x0009	/* Show/Hide main window	      */
 #define ICQCMD_CLOSELOG	0x000A	/* Close User's log/history files     */

/*----------------------------------------------------------------------------*/

#pragma pack(1)

/* Post message command block */
#define MAX_MSG_SIZE 2048
typedef struct _msgcmd
{
   /* Regular shareMem block */
   USHORT       ret;                    /* pwICQ return code            */
   UCHAR        cmd;                    /* pwICQ request code           */
                                        /* Always 0x0B for this request */

   /* Message block */
   USHORT       type;                   /* Command type         */
   #define MSGCMD_GETUSERINFO   0x01
   #define MSGCMD_ADDUSER       0x02
   #define MSGCMD_SENDMESSAGE   0x03

   ULONG        to;                     /* Destination UIN      */
   char         NickName[20];           /* User's nickname      */
   char         FirstName[19];          /* User's firstname     */
   char         LastName[40];           /* User's lastname      */
   char         EMail[0x0100];          /* User's E-Mail        */
   char         msgText[MAX_MSG_SIZE];  /* Message text         */
} MSGCMD;

/* Set user status command block */
typedef struct _usrstatcmd
{
   /* Regular shareMem block */
   USHORT       ret;                    /* pwICQ return code            */
   UCHAR        cmd;                    /* pwICQ request code           */
                                        /* Always 0x0C for this request */

   /* Command block */
   ULONG        uin;                    /* User UIN             */
   ULONG        mode;                   /* Desired mode         */
   long         ip[2];                  /* User IP Adresses     */
   int          port;                   /* User port            */

   /* User address, unresolved */
   char         addr[0x0100];           /* hostname:port        */

} USRSTATCMD;

/* Plugin command block prefix (ICQPExternCMD) */
typedef struct _plgcmd
{
   /* Regular shareMem block */
   USHORT       ret;                    /* pwICQ return code            */
   UCHAR        cmd;                    /* pwICQ request code           */
                                        /* Always 0x0D for this request */

   char name[10];                       /* Plugin name, Null Terminated */
} PLGCMD;

/* Send ShareMEM block, wait for answer */
APIRET icqSendMemBlock(const char *, ULONG, char *);

/* Send command with ULONG parms */
APIRET icqSendULongCMD(const char *, char, int, const ULONG *);

/* Send command with STRING parms */
APIRET icqSendUserInfo(const char *, char, ULONG, const char *, const char *, const char *, const char *);

/* Query the current user */
ULONG  icqQueryCurrent(void);

/* Post a MSGCMD struct, wait for response if needed */
APIRET icqSendMsgCmd(ULONG, MSGCMD *, int);

#ifdef PWICQMEM_INCLUDED

   /* Get the pwICQ's shared memory block */
   ICQSHAREMEM *icqGetMemoryBlock(void);

#endif

/* Query the online status */
BOOL icqIsOnline(ULONG);

