/*
 * pwICQDEF.H - Defines/Structures for pwICQ API
 */


#ifndef PWICQDEF_INCLUDED

 #define PWICQDEF_INCLUDED

 #define _PWICQVERSION_ "1.00"

/*---[ Message/Buffer sizes ]---------------------------------------------------*/

 #define MSG_SERVER_SIZE        350   /* Max. send_thru_server message size   */
 #define MSG_DIRECT_SIZE        512   /* Max. send_direct message size        */

 #define MSG_PWICQ_SIZE         512   /* Max. message size to other pwICQ     */
 #define MSG_TEXTEDIT_SIZE     1024   /* Size of the message edit box         */
 #define TCP_PACKET_SIZE       2048   /* Max. size of a TCP packet            */

 #define TCP_PKTSIZE           2048

/* #define MSG_MAXLEN    512 */

/*---[ Menu IDs ]---------------------------------------------------------------*/

 #define MNU_SETMODE    0x00            /* Set mode */
 #define MNU_USER       0x01            /* User popup */
 #define MNU_SYSTEM     0x02            /* pwICQ's button menu */
 #define MNU_SCRIPT     0x03            /* Script list */

 #define MNU_QUANT      0x04

 #define MNU_MSGBUTTON  99              /* Botao na caixa de mensagem */

/*---[ Modes ]------------------------------------------------------------------*/

 #define ICQ_OFFLINE            0xFFFFFFFF       /* Offline */
 #define ICQ_ONLINE             0x00000000       /* Online */
 #define ICQ_AWAY               0x00000001       /* Away */
 #define ICQ_NA                 0x00000005       /* N/A (Extended away) */
 #define ICQ_NA99A              0x00000004
 #define ICQ_BUSY               0x00000010       /* Occupied (Mac) */
 #define ICQ_OCCUPIED           0x00000011       /* Occupied (Urgent MSGs only) */
 #define ICQ_DND                0x00000013       /* Do not Disturb */
 #define ICQ_DND2               0x00000002       /* Do not Disturb (LICQ) */
 #define ICQ_FREECHAT           0x00000020       /* Available for chat */
 #define ICQ_PRIVACY            0x00000100       /* Privacy (Invisible) */

/*---[ Mode Flags ]-------------------------------------------------------------*/

 #define ICQ_STATUS             0x000001FF       /* Mascara para separar online/offline status das flags */
 #define ICQ_WEBPRESENCE        0x00010000       /* Ativa WebPresence */
 #define ICQ_HIDEIP             0x00020000
 #define ICQ_PHONEBUSY          0x00040000
 #define ICQ_BIRTHDAY           0x00080000
 #define ICQ_SOCKS              0x00100000      /* Conexao via SOCKS */
 #define ICQ_WEBSERVER          0x00200000      /* Webserver ativo */

 #define ICQ_KNOWMODES          0x803F01FF      /* Modos que eu ja conheco */

/*---[ API Function types ]-----------------------------------------------------*/

 #ifndef HICQ
    #define HICQ void *
 #endif

 #ifndef HUSER
    #define HUSER USER *
 #endif

 #ifndef ICQFILE
    #define ICQFILE void *
 #endif

/*---[ Structures ]-------------------------------------------------------------*/

 #pragma pack(1)

 typedef struct icqdate
 {
    unsigned short      year;
    char                month;
    char                day;
    char                hour;
    char                minute;
    char                seconds;
 } ICQDATE;

 typedef struct user
 {
    ULONG               uin;             /* UIN */
    ULONG               mode;            /* Status (Online, away, etc) */
    long                ip[2];           /* IP Adresses (reporteds from user and from server) */
    int                 port;            /* TCP port to connect */
    ULONG               flags;           /* Options/Flags */
    ULONG               group;           /* Reserved */
    ULONG               version;         /* Client version */
    short               btmCode;         /* Reserved */
 } USER;

 typedef struct usrmsg                   /* Message on the user's queue */
 {
    USHORT              type;            /* Message type */
    USHORT              tcpFlags;        /* TCP Message Flags */
    /* TCP Message Command Types */
    #define TCP_MSG_AUTO         0x0000  /* TCP message ACK and Auto-Reply              */
    #define TCP_MSG_REAL         0x0010  /* TCP message                                 */
    #define TCP_MSG_LIST         0x0020  /* TCP message sent to the contact list        */
    #define TCP_MSG_URGENT       0x0040  /* TCP message sent urgently                   */

    /* The following are flags and are OR'd with the above Command Types */
    #define TCP_MSGF_S_INVISIBLE 0x0080  /* The message sender is Invisible             */
    #define TCP_MSGF_S_AWAY      0x0100  /* The message sender is Away                  */
    #define TCP_MSGF_S_OCCUPIED  0x0200  /* The message sender is Occupied              */
    #define TCP_MSGF_S_NA        0x0800  /* The message sender is Not Available         */
    #define TCP_MSGF_S_DND       0x1000  /* The message sender is in Do Not Disturb     */

    ICQDATE             date;            /* Date/Time of the message */
    long                seq;             /* Sequence number (direct TCP messages only) */
    HWND                hwnd;            /* Message Window */

 } USRMSG;

 typedef struct sysmsg                  /* Message on the system's queue */
 {
    long                uin;            /* The sender    */
    USRMSG              msg;            /* The message   */
 } SYSMSG;

 typedef struct dlginfo                 /* Dialog helper */
 {
    HWND        hwnd;                   /* Window's handle */
    HICQ        ctl;                    /* ICQ handle */
    long        uin;                    /* Owner's of the dialog UIN */
    HUSER       usr;                    /* Owner's of the dialog handle */
    HINI        ini;                    /* Handle of the pwICQ INI file */
    char        id[11];                 /* Window's Identification */
    HPOINTER    icon;                   /* Window's Icon */
    USHORT      type;                   /* Dialog's type */

    /* Work areas for the Dialog's message processor */
    long        flags;                  /* Generic flags */
    long        lValue;                 /* Generic Value */
    char        temp[0x0100];           /* Temporary buffer */

 } DLGINFO;


/*---[ Message types ]----------------------------------------------------------*/

 #define MSG_NORMAL     0x0001          /* Normal */
 #define MSG_CHAT       0x0002          /* Chat Request */
 #define MSG_FILE       0x0003          /* File transfer request */
 #define MSG_URL        0x0004          /* URL */
 #define MSG_REQ        0x0006          /* Authorization request */
 #define MSG_REFUSED    0x0007          /* Authorization denied */
 #define MSG_AUTHORIZE  0x0008          /* Authorized */
 #define MSG_SYSREQ     0x0009          /* System request */
 #define MSG_ADD        0x000C          /* "You were added" */
 #define MSG_PAGER      0x000D          /* ICQ-Pager message */
 #define MSG_MAIL       0x000E          /* Mail Express */
 #define MSG_CONTACT    0x0013          /* Contact-list message */

/*---[ System/User event codes ]------------------------------------------------*/

 #define EVT_ONLINE     0x00000001      /* Usuario/sistema on-line */
 #define EVT_OFFLINE    0x00000002      /* Usuario/sistema off-line */
 #define EVT_STATUS     0x00000003      /* Mudanca de status */
 #define EVT_CONNECT    0x00000004      /* Iniciou processo de conexao */
 #define EVT_RECV       0x00000005      /* Pacote recebido */
 #define EVT_SENT       0x00000006      /* Pacote enviado */
 #define EVT_CONNCT     0x00000007      /* Conexao recebida */
 #define EVT_DISCONNCT  0x00000008      /* Conexao encerrada */
 #define EVT_LOGONOK    0x00000009      /* Processo de logon completo */
 #define EVT_STARTUP    0x0000000A      /* Sistema iniciando */
 #define EVT_CONTACT    0x0000000B      /* Contact-list confirmada */
 #define EVT_GOAWAY     0x0000000C      /* Servidor pediu p'ra sair */
 #define EVT_RETRY      0x0000000D      /* Servidor pediu p'ra tentar de novo */
 #define EVT_NOTFOUND   0x0000000E      /* Usuario nao encontrado */
 #define EVT_FOUND      0x0000000F      /* Informacao de usuario recebida */
 #define EVT_ENDMSG     0x00000010      /* Fim das mensagems armazenadas */
 #define EVT_ENDSEARCH  0x00000011      /* Pesquisa terminada */
 #define EVT_USERFOUND  0x00000012      /* Usuario encontrado (Pesquisa) */
 #define EVT_REJECTED   0x00000013      /* Rejeitado pelo servidor */
 #define EVT_CONTACTED  0x00000014      /* Servidor foi contactado */
 #define EVT_NETFAIL    0x00000015      /* Falha na rede */
 #define EVT_TCPREJECT  0x00000016      /* Conexao TCP rejeitada */
 #define EVT_BEEP       0x00000017      /* Beep */
 #define EVT_PARM       0x00000018      /* Parametro foi mudado */
 #define EVT_MESSAGE    0x00000019      /* Mensagem recebida */

/*---[ User flags ]-------------------------------------------------------------*/

 #define USR_INLIST     0x00000001      /* Usuario esta na lista? */
 #define USR_BLINK      0x00000002      /* Usuario esta piscando? */
 #define USR_PENDENT    0x00000004      /* Usuario tem eventos pendentes?? */
 #define USR_WAITING    0x00000008      /* Aguardando autorizacao */
 /* #define USR_PWICQ      0x00000010 */     /* Usuario utiliza o pwICQ, posso usar funcoes extras */
 #define USR_REFUSED    0x00000020      /* Usuario recusou a anexacao */
 #define USR_DELETED    0x00000040      /* Usuario foi deletado */
 #define USR_IGNORE     0x00000080      /* Ignorar mensagems do usuario */
 #define USR_AUTOOPEN   0x00000100      /* Abre mensagems do usuario automaticamente */
 #define USR_REFRESH    0x00000200      /* Atualizar dados do usuario */
 #define USR_AUTHORIZE  0x00000400      /* Usuario tem que autorizar anexacao */
 #define USR_NOINFO     0x00000800      /* Nao foi possivel pesquisar o usuario */
 #define USR_LOGMSG     0x00001000      /* Gravar log de todas as mensagems do usuario */
 #define USR_ACT        0x00002000      /* Houve atividade no usuario */
 #define USR_INVISIBLE  0x00004000      /* Usuario esta na minha 'invisible list' */
 #define USR_VISIBLE    0x00008000      /* Usuario esta na minha 'visible list' */
 #define USR_REXX       0x00010000      /* Habilita scripts Rexx para o usuario */
 #define USR_SEARCH     0x00020000      /* Existe uma pesquisa ativa para o usuario */
 #define USR_NOAWAY     0x00040000      /* Ignora mensagems de AWAY do usuario */
 #define USR_NOSNDAWAY  0x00080000      /* Nao envia mensagems de AWAY ao usuario */
 #define USR_HISTORY    0x00100000      /* Usuario usa arquivo de historico */
 #define USR_ACCEPT     0x00200000      /* Always accept files */
 #define USR_AUTORESUME 0x00400000      /* Always resume */
 #define USR_DIRECT     0x00800000      /* Direct connected (Offline) */
 #define USR_WINPOS     0x01000000      /* Save user's window positions */
 #define USR_FILEDIR    0x02000000      /* Receive files in user's folder */

 #define USR_HIDE       0x40000000      /* Nao mostra na contact-list */



#endif

